/* -*- c++ -*- */
/* 
 * This file is part of dkim-verify
 * Copyright (C) 2008 Erik Lotspeich <erik@lotspeich.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation: version 2
 * of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 */

#ifndef DKIM_VERIFY_H
#define DKIM_VERIFY_H 1

#include <iostream>
#include <string>
#include <list>

#include <liblot_logging.h>
#include <liblot_args.h>

#include "dkim-config.h"

#if defined(__GNU_LIBRARY__) && !defined(_SEM_SEMUN_UNDEFINED)
/* union semun is defined by including <sys/sem.h> */
#else
/* according to X/OPEN we have to define it ourselves */
union semun {
        int val;                  /* value for SETVAL */
        struct semid_ds *buf;     /* buffer for IPC_STAT, IPC_SET */
        unsigned short *array;    /* array for GETALL, SETALL */
                                  /* Linux specific part: */
	struct seminfo *__buf;    /* buffer for IPC_INFO */
};
#endif

namespace dkim_vrfy {
	class main_app;
	enum proj_ids {
		ID_QUIT_TH_SYNC = 'q',
	};

	namespace args {
		class opt_H;
		class opt_P;
		class opt_V;
		class opt_nf;
	}
}

class dkim_vrfy::main_app {
	static const std::string VERSION;
	static const std::string COPYRIGHT_YEAR;
	static const std::string AUTHOR;
	static const std::string AUTHOR_EMAIL;
	static const std::string LICENSE;
	static main_app *pinstance;
	std::string profile;
	dkim_vrfy::config::cfg_access *cfga;

	/* Don't allow copies */
	main_app(const main_app &);
	main_app &operator=(const main_app &);
protected:
	main_app();
public:
	static const std::string SYSLOG_NAME;

	~main_app() {
		delete cfga;
		cfga = 0;
	}

	liblot::logging::syslogpp sl;

	static main_app *instance();

	dkim_vrfy::config::cfg_access *get_cfga() { return cfga; }

	void set_profile(const std::string &p) { profile = p; }
	const std::string &get_profile() const { return(profile); }

	std::string get_cfg_value(const std::string &) const;

	static void print_title();
	static void print_help();
};

class dkim_vrfy::args::opt_H : public liblot::args::opt_handler {
public:
        void handle(const liblot::args::option &);
};

class dkim_vrfy::args::opt_P : public liblot::args::opt_handler {
public:
        void handle(const liblot::args::option &);
};

class dkim_vrfy::args::opt_V : public liblot::args::opt_handler {
public:
        void handle(const liblot::args::option &);
};

class dkim_vrfy::args::opt_nf : public liblot::args::opt_handler {
	std::list<std::string> *pa;
public:
	opt_nf(std::list<std::string> *p) : pa(p) {}

        void handle(const liblot::args::option &opt) {
		pa->push_back(opt.get_opt());
	}
};

#endif // DKIM_VERIFY_H
