/* -*- c++ -*- */
/* 
 * This file is part of liblot
 * Copyright (C) 2008 Erik Lotspeich <erik@lotspeich.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation: version 2
 * of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#ifndef LIBLOT_ARGS_H
#define LIBLOT_ARGS_H 1

#include <string>
#include <map>

#include "liblot_exception.h"

namespace liblot {
	namespace args {
		class opt_handler;
		class option;
		class getlopt;
	}
}

class liblot::args::opt_handler {
public:
	virtual ~opt_handler() {}

	virtual void handle(const option &) = 0;
};

class liblot::args::option {
	bool takes_arg;
	std::string opt;
	std::string optarg;
	opt_handler *handler;

	/* Don't allow copies */
	option(const option &);
	option &operator=(const option &);
public:
	option() : takes_arg(false), handler(0) {}
	option(bool t) : takes_arg(t), handler(0) {}
	option(const std::string &o) :
		takes_arg(false), opt(o), handler(0) {}
	option(const std::string &o, const std::string &oarg) :
		takes_arg(true), opt(o), optarg(oarg), handler(0) {}

	bool get_takes_arg() const { return(takes_arg); }
	void set_takes_arg(bool a) { takes_arg = a; }
	std::string get_opt() const { return(opt); }
	void set_opt(const std::string &o) { opt = o; }
	std::string get_optarg() const { return(optarg); }
	void set_optarg(const std::string &oarg) { optarg = oarg; }

	void set_handler(opt_handler *h) { handler = h; }
	void handle_opt() {
		if (handler) {
			handler->handle(*this);
		} else {
			throw liblot::exception::
				app_notice("No handler set");
		}
	}

	bool operator<(const option &rhs) const {
		return(opt < rhs.get_opt());
	}
};

class liblot::args::getlopt {
	int ah_argc;
	char **ah_argv;

	/* Registered options */
	std::map<std::string, option *> reg_opts;

	/* opt-not-found handler */
	opt_handler *opth_nf;

	/* Don't allow copies */
	getlopt(const getlopt &);
	getlopt &operator=(const getlopt &);
public:
	getlopt() : ah_argc(0), ah_argv(0), opth_nf(0) {}
	getlopt(int c, char **v) : ah_argc(c), ah_argv(v) {}

	void register_opt(const std::string &o, option *opt) {
		reg_opts[o] = opt;
	}
	void register_nf_handler(opt_handler *opth) {
		opth_nf = opth;
	}
	void process_args();
};

#endif // LIBLOT_ARGS_H
