/* -*- c++ -*- */
/* 
 * This file is part of liblot
 * Copyright (C) 2008 Erik Lotspeich <erik@lotspeich.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation: version 2
 * of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 */

#ifndef LIBLOT_EXCEPTION_H
#define LIBLOT_EXCEPTION_H 1

#include <iostream>
#include <string>
#include <exception>
#include <stdexcept>

namespace liblot {
	namespace exception {
		class app_exception;
		class app_error;
		class app_notice;
		class app_range;
	}
}

/* Exception handlers */
class liblot::exception::app_exception : public std::exception {
public:
        virtual ~app_exception() throw() {}

	virtual const char *what() const throw() = 0;

        virtual std::string get_str() const = 0;
	virtual std::string get_id() const = 0;
	virtual void print() const {
		std::cerr << get_id() << ": " << get_str() << std::endl;
	};
};

std::ostream &operator<<(std::ostream &,
			 const liblot::exception::app_exception &);

class liblot::exception::app_error : public app_exception,
				     public std::runtime_error {
public:
        app_error(const std::string &s) : runtime_error(s) {}
	virtual ~app_error() throw() {}

	virtual const char *what() const throw() {
		return(std::runtime_error::what());
	}

	std::string get_str() const { return(what()); }
	std::string get_id() const { return("Error"); }
};

class liblot::exception::app_notice : public app_exception,
				      public std::runtime_error {
public:
        app_notice(const std::string &s) : runtime_error(s) {}
	virtual ~app_notice() throw() {}

	virtual const char *what() const throw() {
		return(std::runtime_error::what());
	}

	std::string get_str() const { return(what()); }
	std::string get_id() const { return("Notice"); }
};

class liblot::exception::app_range : public app_exception,
			             public std::out_of_range {
public:
	app_range(const std::string &s) : out_of_range(s) {}
	virtual ~app_range() throw() {}

	virtual const char *what() const throw() {
		return(std::out_of_range::what());
	}

	std::string get_str() const { return(what()); }
	std::string get_id() const { return("Range"); }
};

#endif // LIBLOT_EXCEPTION_H
