/* -*- c++ -*- */
/* 
 * This file is part of liblot
 * Copyright (C) 2008 Erik Lotspeich <erik@lotspeich.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation: version 2
 * of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 */

#ifndef LIBLOT_LOGGING_H
#define LIBLOT_LOGGING_H 1

#include <iostream>
#include <string>
#include <cstring>

#include <syslog.h>

#ifndef ERROR
#define ERROR -1
#endif

namespace liblot {
	namespace logging {	
		struct IDENT;
		struct FACILITY;
		struct OPTION;
		struct PRIORITY;
		class syslogpp;
	}
}

struct liblot::logging::IDENT {
        explicit IDENT(std::string s) : val(s) {}
	explicit IDENT(const char *s) : val(s) {}
        std::string val;
};

struct liblot::logging::OPTION {
        explicit OPTION(int s) : val(s) {}
        int val;
};

struct liblot::logging::FACILITY {
        explicit FACILITY(int s) : val(s) {}
        int val;
};

struct liblot::logging::PRIORITY {
        explicit PRIORITY(int s) : val(s) {}
        int val;
};

/* C++ syslog implementation */
class liblot::logging::syslogpp {
	std::string ident;
	bool openlog_called;
public:
	syslogpp() : openlog_called(false) {}
	syslogpp(const IDENT &i, const OPTION &opt,
		 const FACILITY &fac) : ident(i.val), openlog_called(true) {
		openlog(ident.c_str(), opt.val, fac.val);
	}
	syslogpp(const IDENT &i) : ident(i.val), openlog_called(true) {
		openlog(ident.c_str(), LOG_PID, LOG_USER);
	}
	~syslogpp() { if (openlog_called) closelog(); }

	std::string get_ident_current() const { return(ident); }
	void set_ident_current(const std::string &s) { ident = s; }
	void open(const OPTION &opt, const FACILITY &fac) {
		openlog(ident.c_str(), opt.val, fac.val);
		openlog_called = true;
	}

	void log(const PRIORITY &pri, const std::string &msg) const {
		syslog(pri.val, msg.c_str());
	}
};

#endif // LIBLOT_LOGGING_H
