/* -*- c++ -*- */
/* 
 * This file is part of liblot
 * Copyright (C) 2008 Erik Lotspeich <erik@lotspeich.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation: version 2
 * of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 */

#ifndef LIBLOT_TEXT_H
#define LIBLOT_TEXT_H 1

#include <string>

namespace liblot {
	namespace text {
		enum eol_delim_style {
			EOL_DELIM_LF,
			EOL_DELIM_CRLF,
		};

		class eol_helper;
	}
}

class liblot::text::eol_helper {
	static const char CR;
	static const char LF;
	static const std::string CRLF;

	eol_delim_style style;
	std::string delim;
public:	
	eol_helper(eol_delim_style e = EOL_DELIM_LF);

	std::string get_delim() const { return(delim); }
	std::string get_delim(int) const;
	eol_delim_style get_delim_style() const { return(style); }

	std::string to_nl(const std::string &) const;
	std::string to_style(const std::string &, eol_delim_style) const;

	static std::string nl_to_crlf(const std::string &);
	static std::string crlf_to_nl(const std::string &);
};

#endif // LIBLOT_TEXT_H
