/* -*- c++ -*- */
/* 
 * This file is part of liblot
 * Copyright (C) 2008 Erik Lotspeich <erik@lotspeich.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation: version 2
 * of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include <sstream>

#include <cstring>

#include <errno.h>
#include <unistd.h>
#include <limits.h>

#include "liblot_exception.h"
#include "liblot_network.h"

std::string liblot::network::hostname::get_hostname()
{
	char hname[HOST_NAME_MAX];

	if (gethostname(hname, HOST_NAME_MAX) == -1) {
		std::ostringstream msg;

		msg << "gethostname: " << strerror(errno);

		throw liblot::exception::app_error(msg.str());
	}

	hname[HOST_NAME_MAX - 1] = '\0';

	return(hname);
}

void liblot::network::hostname::set_hostname(const std::string &hname)
{
	using namespace liblot::exception;

	if (hname.empty()) {
		throw app_notice("No hostname to set");
	}

	if (sethostname(hname.c_str(), hname.size()) == -1) {
		std::ostringstream msg;

		msg << "sethostname: " << strerror(errno);

		throw app_error(msg.str());
	}
}
