/* -*- c++ -*- */
/* 
 * This file is part of liblot
 * Copyright (C) 2008 Erik Lotspeich <erik@lotspeich.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation: version 2
 * of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 */

#include <string>
#include <sstream>

#include "liblot_text.h"
#include "liblot_exception.h"

const char liblot::text::eol_helper::CR = '\r';
const char liblot::text::eol_helper::LF = '\n';
const std::string liblot::text::eol_helper::CRLF("\r\n");

liblot::text::eol_helper::eol_helper(eol_delim_style e) : style(e)
{
	switch (e) {
		case EOL_DELIM_CRLF:
			delim.assign(CRLF);
			break;
		case EOL_DELIM_LF:
		default:
			delim.assign(1, LF);
	}
}

std::string liblot::text::eol_helper::get_delim(int n) const
{
	if (n <= 0) throw liblot::exception::
		app_range("Delimiter repeat value must be greater than 0");

	std::string d;

	for (int i = 0; i < n; i++) {
		d.append(get_delim());
	}

	return(d);
}

std::string
liblot::text::eol_helper::to_nl(const std::string &str) const
{
	using namespace liblot::exception;

	if (style == EOL_DELIM_LF) return(str);

	std::string converted_str;

	switch (style) {
	case EOL_DELIM_CRLF:
		converted_str = crlf_to_nl(str);
		break;
	default:
		throw app_notice("Unknown EOL delimiter style");
	}

	return(converted_str);
}

std::string
liblot::text::eol_helper::to_style(const std::string &str,
				   eol_delim_style src_style) const
{
	using namespace liblot::exception;

	if (style == src_style) return(str);

	std::string converted_str;

	/* Assumption: if it's not one, it's the other */
	switch(style) {
	case EOL_DELIM_LF:
		converted_str = crlf_to_nl(str);
		break;
	case EOL_DELIM_CRLF:
		converted_str = nl_to_crlf(str);
		break;
	default:
		throw app_notice("Unknown EOL delimiter style");	
	}

	return(converted_str);
}

/* Change '\n' to '\r\n' */
std::string liblot::text::eol_helper::nl_to_crlf(const std::string &str)
{
	std::istringstream is(str);
	std::ostringstream os;
	std::string line;

	while (is.good()) {
		std::getline(is, line);

		os << line << CRLF;
	}	

	return(os.str());
}

/* Change '\r\n' to '\n' */
std::string liblot::text::eol_helper::crlf_to_nl(const std::string &s)
{
        size_t pos = 0;

	std::string str(s);
	
	/* TODO: Fix performance; this algorithm is terrible */
        while ((pos = str.find(CRLF, pos)) != std::string::npos) {
                str.erase(pos, 1);
        }

	return(str);
}
